package gs.config

import cats.effect.Sync

/** Interface for loading configuration.
  */
trait Configuration[F[_]]:
  def getValue[A: Configurable](key: ConfigKey[A]): F[Either[ConfigError, A]]

object Configuration:

  /** Start building a new [[AuditedConfiguration]].
    *
    * @return
    *   Fluent builder for the [[AuditedConfiguration]] class.
    */
  def audited[F[_]: Sync]: AuditedConfiguration.Builder[F] =
    AuditedConfiguration.builder[F]

  /** Create a new [[AuditedConfiguration]] that reads from the environment.
    *
    * @return
    *   The new [[Configuration]].
    */
  def auditedEnvironmentOnly[F[_]: Sync]: F[Configuration[F]] =
    audited[F].withEnvironmentSource().build()

end Configuration
